// -*- C++ -*-
// ACL:license
// ----------------------------------------------------------------------
// This software and ancillary information (herein called "SOFTWARE")
// called POOMA (Parallel Object-Oriented Methods and Applications) is
// made available under the terms described here.  The SOFTWARE has been
// approved for release with associated LA-CC Number LA-CC-98-65.
// 
// Unless otherwise indicated, this SOFTWARE has been authored by an
// employee or employees of the University of California, operator of the
// Los Alamos National Laboratory under Contract No. W-7405-ENG-36 with
// the U.S. Department of Energy.  The U.S. Government has rights to use,
// reproduce, and distribute this SOFTWARE. The public may copy, distribute,
// prepare derivative works and publicly display this SOFTWARE without 
// charge, provided that this Notice and any statement of authorship are 
// reproduced on all copies.  Neither the Government nor the University 
// makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this SOFTWARE.
// 
// If SOFTWARE is modified to produce derivative works, such modified
// SOFTWARE should be clearly marked, so as not to confuse it with the
// version available from LANL.
// 
// For more information about POOMA, send e-mail to pooma@acl.lanl.gov,
// or visit the POOMA web page at http://www.acl.lanl.gov/pooma/.
// ----------------------------------------------------------------------
// ACL:license

//-----------------------------------------------------------------------------
// Tests LinearExtrapolateFaceBC
//-----------------------------------------------------------------------------

#include "Pooma/Fields.h"

#include <iostream>

int main(int argc, char *argv[])
{
  Pooma::initialize(argc, argv);

  // Useful ingredients for constructing and initializing Fields below:

  // Vertex domain:
  int nVerts = 4;
  Interval<1> I(nVerts), J(nVerts);
  Interval<2> domain(I,J);

  // Construct 2D mesh, zero-origin, unit-spacing:
  typedef UniformRectilinearMesh<2> Mesh_t;
  Mesh_t mesh(domain);

  // Construct 2D Cell-centered Geometry, with 1 guard layer:
  GuardLayers<2> gl(2);
  typedef DiscreteGeometry<Cell, Mesh_t> Geometry_t;
  Geometry_t geometry(mesh, gl);

  // --------------------------------------------------------------------------
  // Set a LinearExtrapolateFaceBC on one face of a scalar Field:

  // Create scalar Field (Field<2,double,....>):
  Field<Geometry_t, double> a(geometry);

  // First assign total domain to a flag value, to make sure guards are set by
  // BC below:
  a(a.totalDomain()) = 999.0;

  // Assign physical domain elements to helpful values for diagnosing the BC
  // application results:
  for (int j = a.physicalDomain()[1].min(); 
       j <= a.physicalDomain()[1].max(); j++) {
    for (int i = a.physicalDomain()[0].min(); 
	 i <= a.physicalDomain()[0].max(); i++) {
      a(i,j) = i + j;
    }
  }
  std::cout << "~~~~~a~~~~~~~" << std::endl
            << a.all() << std::endl << std::endl;

  // Use a BcondList, for convenience:
  BCondList bcCell;

  // Make LinearExtrapolateFaceBC BCs, and add to the BCondList:
  bcCell.addBoundaryCondition(a, LinearExtrapolateFaceBC(1)); // High X face

  Range<1> s(-1,nVerts - 1);
  Range<2> ss(s, s);
  
  bcCell.notifyPreRead(ss);
  std::cout << a.all() << std::endl;
  

  // --------------------------------------------------------------------------
  // Set LinearExtrapolateFaceBC on all faces of a vector Field:

  Field<Geometry_t, Vector<2> > b(geometry);

  // First assign total domain to a flag value, to make sure guards are set by
  // BC below:
  b(b.totalDomain()) = Vector<2>(999);

  // Assign physical domain elements to helpful values for diagnosing the BC
  // application results:
  for (int j = a.physicalDomain()[1].min(); 
       j <= a.physicalDomain()[1].max(); j++) {
    for (int i = a.physicalDomain()[0].min(); 
	 i <= a.physicalDomain()[0].max(); i++) {
      for (int d = 0; d < 2; d++) {
	b(i,j)(d) = i + j + d;
      }
    }
  }
  std::cout << "~~~~~~~~~~~~b~~~~~~~~~~" << std::endl << b.all()
	    << std::endl << std::endl;

  // Use a BcondList, for convenience:
  BCondList vbc;

  // Make LinearExtrapolateFaceBC BCs, and add to the BCondList
  vbc.addBoundaryCondition(b, LinearExtrapolateFaceBC(0)); // Low X face:
  vbc.addBoundaryCondition(b, LinearExtrapolateFaceBC(1)); // High X face:
  vbc.addBoundaryCondition(b, LinearExtrapolateFaceBC(2)); // Low Y face:
  vbc.addBoundaryCondition(b, LinearExtrapolateFaceBC(3)); // High Y face:

  vbc.notifyPreRead(ss);
  std::cout << b.all() << std::endl;
  
  Pooma::finalize();
  return 0;
}

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: TestLinExtrapFaceBC.cpp,v $   $Author: swhaney $
// $Revision: 1.13 $   $Date: 2000/03/07 13:16:14 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
