// -*- C++ -*-
// ACL:license
// ----------------------------------------------------------------------
// This software and ancillary information (herein called "SOFTWARE")
// called POOMA (Parallel Object-Oriented Methods and Applications) is
// made available under the terms described here.  The SOFTWARE has been
// approved for release with associated LA-CC Number LA-CC-98-65.
// 
// Unless otherwise indicated, this SOFTWARE has been authored by an
// employee or employees of the University of California, operator of the
// Los Alamos National Laboratory under Contract No. W-7405-ENG-36 with
// the U.S. Department of Energy.  The U.S. Government has rights to use,
// reproduce, and distribute this SOFTWARE. The public may copy, distribute,
// prepare derivative works and publicly display this SOFTWARE without 
// charge, provided that this Notice and any statement of authorship are 
// reproduced on all copies.  Neither the Government nor the University 
// makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this SOFTWARE.
// 
// If SOFTWARE is modified to produce derivative works, such modified
// SOFTWARE should be clearly marked, so as not to confuse it with the
// version available from LANL.
// 
// For more information about POOMA, send e-mail to pooma@acl.lanl.gov,
// or visit the POOMA web page at http://www.acl.lanl.gov/pooma/.
// ----------------------------------------------------------------------
// ACL:license

#ifndef POOMA_DOMAIN_SLICE_INTERVAL_H
#define POOMA_DOMAIN_SLICE_INTERVAL_H

//-----------------------------------------------------------------------------
// Class:
// SliceInterval<int,int>
//-----------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Overview: 
//
// Slice domain objects are domains which have N dimensions worth of 1D
// domain data, but really represent the result of taking an M-dimensional
// slice (M < N) of another N dimensional domain.
//
// SliceInterval<N,M> is is basically an array of N Interval<1> objects,
// but it also knows that only M of these are full domains, and that N-M
// domains are actually referring to single points.  You can retrieve all
// N 1D domains as a normal Interval<N> object, or the smaller slice domain
// as an Interval<M> object.
//
// SliceInterval defers most of its implementation to the
// SliceDomain<DomainTraits<SliceInterval>> base class.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Typedefs:
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes:
//-----------------------------------------------------------------------------

#include "Domain/DomainTraits.SliceInterval.h"
#include "Domain/SliceDomain.h"
#include "Utilities/NoInit.h"
#include "Utilities/PAssert.h"

//-----------------------------------------------------------------------------
// Forward Declarations:
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//
// Full Description of SliceInterval:
//
// SliceInterval<N,M> is a special form of domain object which stores a
// total domain and a slice domain, both as Interval<N> or <M> objects.
// It is a subclass of SliceDomain, which is a general base class for all
// sliced domain objects.  A sliced domain stores a total domain and a
// subset of the total domain, called the slice domain, which has a number
// of dimensions M < less than the total number of dimensions N.  Other
// than being a specialization to Interval domains, SliceInterval has
// the exact same interface as described in SliceDomain.
//
// SliceInterval has only a default constructor and a copy constructor.
// The default constructor initializes the two internal domain objects with
// their default constructors.  It is generally initialized with the
// 'fillSlice' or 'combineSlice' methods in NewDomain.h, which combine
// other non-slice domains into a SliceDomain subclass.
//
// A traits class DomainTraits<SliceInterval<N,M>> is used to provide
// information to the rest of the code about SliceInterval.  The traits
// for SliceDomain subclasses are given in DomainTraits.SliceInterval.h and
// similar files.
//
// There is no specialization to 1D for SliceInterval, since it is not
// needed.  Typically, after a SliceInterval is created by whatever
// means, you call either 'getTotalDomain()' or 'getSliceDomain()' to
// retrieve a reference to the relevant domain, and then use that domain
// as normal.  getTotalDomain() returns an Interval<N> ref, while
// getSliceDomain() returns an Interval<M> ref.
//
//-----------------------------------------------------------------------------

template<int Dim, int SliceDim>
class SliceInterval
  : public SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > >
{
public:
  //
  // Constructors.
  //

  // default constructor : initialize to an empty slice domain
  SliceInterval() { }

  // skip initialization constructor ... if a Pooma::NoInit object
  // is provided, skip initialization
  SliceInterval(const Pooma::NoInit &d)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > >(d) { }

  // copy constructor ... base class does all the work
  SliceInterval(const SliceInterval<Dim,SliceDim> &nd)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > >(nd) {
  }

  // constructors to create slices directly.
  
  template <class Base, class D1, class D2>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain2<D1,D2> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2);
  }

  template <class Base, class D1, class D2, class D3>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2, const D3 &d3)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain3<D1,D2,D3> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2, d3);
  }

  template <class Base, class D1, class D2, class D3, 
            class D4>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2, const D3 &d3, 
                const D4 &d4)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain4<D1,D2,D3,D4> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2, d3, d4);
  }

  template <class Base, class D1, class D2, class D3, 
            class D4, class D5>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2, const D3 &d3, 
                const D4 &d4, const D5 &d5)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain5<D1,D2,D3,D4,D5> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2, d3, d4, d5);
  }

  template <class Base, class D1, class D2, class D3, 
            class D4, class D5, class D6>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2, const D3 &d3, 
                const D4 &d4, const D5 &d5, const D6 &d6)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain6<D1,D2,D3,D4,D5,D6> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2, d3, d4, d5, d6);
  }

  template <class Base, class D1, class D2, class D3, 
            class D4, class D5, class D6, class D7>
  SliceInterval(const Base &baseDomain, const D1 &d1, const D2 &d2, const D3 &d3, 
                const D4 &d4, const D5 &d5, const D6 &d6, const D7 &d7)
    : SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > > (
        Pooma::NoInit())
  {
      typedef NewDomain7<D1,D2,D3,D4,D5,D6,D7> NewDomain_t;
      CTAssert(NewDomain_t::SliceType_t::dimensions == dimensions);
      CTAssert(NewDomain_t::SliceType_t::sliceDimensions == sliceDimensions);
      NewDomain_t::fillSlice(*this, baseDomain, d1, d2, d3, d4, d5, d6, d7);
  }

  //
  // Destructor.  For this class there is nothing to do.
  //

  ~SliceInterval() { }

  //
  // operator= ... just use the op= in the base class
  //

  SliceInterval<Dim,SliceDim> &
    operator=(const SliceInterval<Dim,SliceDim> &nd) {
      SliceDomain<DomainTraits<SliceInterval<Dim,SliceDim> > >::operator=(nd);
      return *this;
  }

protected:

private:

};


//////////////////////////////////////////////////////////////////////

#endif     // POOMA_DOMAIN_SLICE_INTERVAL_H

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: SliceInterval.h,v $   $Author: jac $
// $Revision: 1.8 $   $Date: 2000/06/27 01:58:28 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
